
/* Chrysalide - Outil d'analyse de fichiers binaires
 * pending.h - prototypes pour la consolidation de correspondances partielles
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_MATCHES_PENDING_H
#define _ANALYSIS_SCAN_MATCHES_PENDING_H


#include "../../content.h"



/* Couverture d'une correspondance */
typedef struct _match_area_t
{
    phys_t start;                           /* Point de départ             */
    phys_t length;                          /* Taille de la zone couverte  */

} match_area_t;

/* Suivi de correspondances */
typedef struct _pending_matches_t
{
    match_area_t *areas;                    /* Zones couvertes             */
    size_t allocated;                       /* Nombre d'allocations        */
    size_t used;                            /* Nombre de zones             */

    bool initialized;                       /* Etat du suivi               */

} pending_matches_t;


/* Initialise une structure de consolidation de correspondances. */
void init_pending_matches(pending_matches_t *);

/* Libère la mémoire utilisée par une consolidation. */
void exit_pending_matches(pending_matches_t *);

#define are_pending_matches_initialized(pm) pm->initialized

#define set_pending_matches_initialized(pm) pm->initialized = true

/* Détermine la zone de correspondance idéale pour complément. */
bool find_target_in_pending_matches(pending_matches_t *, phys_t, size_t *);

/*  Ajoute au suivi la définition d'une nouvelle correspondance. */
void add_pending_matches(pending_matches_t *, phys_t, phys_t);

/* Etend une zone couverte dans le suivi des correspondances. */
void extend_pending_matches(pending_matches_t *, size_t, phys_t);

/* Retire une correspondance finalement non établie du suivi. */
void remove_pending_matches(pending_matches_t *, size_t);



#endif  /* _ANALYSIS_SCAN_MATCHES_PENDING_H */
