
/* Chrysalide - Outil d'analyse de fichiers binaires
 * token.c - bribes de recherche textuelle
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "token.h"


#include <assert.h>


#include "token-int.h"



/* Initialise la classe des bribes de recherche textuelle. */
static void g_string_token_class_init(GStringTokenClass *);

/* Initialise une instance de bribe de recherche textuelle. */
static void g_string_token_init(GStringToken *);

/* Supprime toutes les références externes. */
static void g_string_token_dispose(GStringToken *);

/* Procède à la libération totale de la mémoire. */
static void g_string_token_finalize(GStringToken *);



/* Indique le type défini pour une bribe de recherche textuelle. */
G_DEFINE_TYPE(GStringToken, g_string_token, G_TYPE_SEARCH_PATTERN);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des bribes de recherche textuelle.      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_string_token_class_init(GStringTokenClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GSearchPatternClass *pattern;           /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_string_token_dispose;
    object->finalize = (GObjectFinalizeFunc)g_string_token_finalize;

    pattern = G_SEARCH_PATTERN_CLASS(klass);

    //pattern->prepare = (prepare_pattern_fc)g_string_token_prepare;
    //pattern->analyze = (analyze_pattern_fc)g_string_token_analyze;
    //pattern->count = (count_pattern_matchs_fc);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : token = instance à initialiser.                              *
*                                                                             *
*  Description : Initialise une instance de bribe de recherche textuelle.     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_string_token_init(GStringToken *token)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : token = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_string_token_dispose(GStringToken *token)
{
    G_OBJECT_CLASS(g_string_token_parent_class)->dispose(G_OBJECT(token));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : token = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_string_token_finalize(GStringToken *token)
{
    G_OBJECT_CLASS(g_string_token_parent_class)->finalize(G_OBJECT(token));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : token   = définition de la bribe à enregistrer.              *
*                context = contexte de l'analyse à mener.                     *
*                backend = moteur de recherche à préchauffer.                 *
*                maxsize = taille max. des atomes (mise en commun optimisée). *
*                                                                             *
*  Description : Inscrit la définition d'un motif dans un moteur de recherche.*
*                                                                             *
*  Retour      : Bilan de l'opération à renvoyer.                             *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_string_token_enroll(GStringToken *token, GScanContext *context, GEngineBackend *backend, size_t maxsize)
{
    bool result;                            /* Statut à retourner          */
    GStringTokenClass *class;               /* Classe de l'instance        */

    assert(g_engine_backend_get_atom_max_size(backend) == maxsize);

    class = G_STRING_TOKEN_GET_CLASS(token);

    result = class->enroll(token, context, backend, maxsize);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : token   = définition de la bribe à manipuler.                *
*                context = contexte de l'analyse à mener.                     *
*                content = accès au contenu brut pour vérifications (optim.)  *
*                matches = suivi des correspondances à consolider.            *
*                                                                             *
*  Description : Transforme les correspondances locales en trouvailles.       *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_string_token_check(const GStringToken *token, GScanContext *context, GBinContent *content, pending_matches_t *matches)
{
    GStringTokenClass *class;               /* Classe de l'instance        */

    class = G_STRING_TOKEN_GET_CLASS(token);

    class->check(token, context, content, matches);

}
